@@ -32,6 +32,25 @@ def flask_stream(request_body: dict) -> tuple[int, list]:
         ]
 
 
+@mock.patch("workflow_server.api.workflow_view.ENABLE_PROCESS_WRAPPER", False)
+def flask_stream_disable_process_wrapper(request_body: dict) -> tuple[int, list]:
+    flask_app = create_app()
+    with flask_app.test_client() as test_client:
+        response = test_client.post("/workflow/stream", json=request_body)
+        status_code = response.status_code
+
+        return status_code, [
+            json.loads(line)
+            for line in response.data.decode().split("\n")
+            if line
+            and line
+            not in [
+                "WAITING",
+                "END",
+            ]
+        ]
+
+
 def code_exec_stream(request_body: dict) -> tuple[int, list]:
     output = io.StringIO()
 
@@ -48,7 +67,7 @@ def code_exec_stream(request_body: dict) -> tuple[int, list]:
     return 200, events
 
 
-@pytest.fixture(params=[flask_stream, code_exec_stream])
+@pytest.fixture(params=[flask_stream, code_exec_stream, flask_stream_disable_process_wrapper])
 def both_stream_types(request):
     return request.param
 
