@@ -7,6 +7,8 @@ from multiprocessing import Process, Queue, set_start_method
 import os
 import pkgutil
 from queue import Empty
+import random
+import string
 import sys
 from threading import Event as ThreadingEvent
 import time
@@ -14,10 +16,12 @@ import traceback
 from uuid import uuid4
 from typing import Any, Dict, Generator, Iterator, Optional, Union, cast
 
-from flask import Blueprint, Response, current_app as app, request, stream_with_context
+from flask import Blueprint, Response, current_app as app, has_request_context, request, stream_with_context
 from pydantic import ValidationError
 from vellum_ee.workflows.display.nodes.get_node_display_class import get_node_display_class
 from vellum_ee.workflows.display.types import WorkflowDisplayContext
+from vellum_ee.workflows.display.workflows import BaseWorkflowDisplay
+from vellum_ee.workflows.server.virtual_file_loader import VirtualFileFinder
 
 from vellum.workflows.exceptions import WorkflowInitializationException
 from vellum.workflows.nodes import BaseNode
@@ -57,6 +61,17 @@ CUSTOM_NODES_DIRECTORY = "vellum_custom_nodes"
 WORKFLOW_INITIATION_TIMEOUT_SECONDS = 60
 
 
+def _events_emission_enabled(context: Union[WorkflowExecutorContext, NodeExecutorContext]) -> bool:
+    flags = context.feature_flags
+    if not (flags and flags.get("vembda-event-emitting-enabled")):
+        return False
+    try:
+        module = importlib.import_module("vellum.workflows.events.emitters.vellum")
+        return hasattr(module, "VellumEmitter")
+    except Exception:
+        return False
+
+
 @bp.route("/stream", methods=["POST"])
 def stream_workflow_route() -> Response:
     data = request.get_json()
@@ -93,6 +108,7 @@ def stream_workflow_route() -> Response:
     headers = {
         "X-Vellum-SDK-Version": vembda_initiated_event.body.sdk_version,
         "X-Vellum-Server-Version": vembda_initiated_event.body.server_version,
+        "X-Vellum-Events-Emitted": str(_events_emission_enabled(context)),
     }
 
     # We can exceed the concurrency count currently with long running workflows due to a knative issue. So here
@@ -111,6 +127,7 @@ def stream_workflow_route() -> Response:
         )
 
     cancel_signal = ThreadingEvent()
+    timeout_signal = ThreadingEvent()
 
     process: Optional[Process] = None
     if ENABLE_PROCESS_WRAPPER:
@@ -119,6 +136,7 @@ def stream_workflow_route() -> Response:
                 executor_context=context,
                 queue=process_output_queue,
                 cancel_signal=cancel_signal,
+                timeout_signal=timeout_signal,
             )
             increment_process_count(1)
         except Exception as e:
@@ -150,6 +168,7 @@ def stream_workflow_route() -> Response:
                     context,
                     disable_redirect=True,
                     cancel_signal=cancel_signal,
+                    timeout_signal=timeout_signal,
                 )
                 yield f"{SPAN_ID_EVENT}:{span_id}"
                 span_id_emitted = True
@@ -201,6 +220,9 @@ def stream_workflow_route() -> Response:
             ):
                 logger.error("Workflow timed out, waiting 5 seconds before ending request...")
                 cancel_signal.set()
+                # We pass this separate signal in so we can get the vembda time_out flag set to true in the vembda
+                # fulfilled event inside of exec. In the future we might have a separate timeout event in wsdk
+                timeout_signal.set()
                 timed_out_time = time.time()
 
             if timed_out_time is not None and timed_out_time + 5 < time.time():
@@ -243,8 +265,7 @@ def stream_workflow_route() -> Response:
 
             try:
                 if ENABLE_PROCESS_WRAPPER:
-                    item = queue.get(timeout=0.1)
-                    event = item
+                    event = queue.get(timeout=0.1)
                 else:
                     event = next(stream_iterator)
             except Empty:
@@ -280,7 +301,7 @@ def stream_workflow_route() -> Response:
             yield "\n"
             yield vembda_initiated_event.model_dump_json()
             yield "\n"
-            for index, row in enumerate(workflow_events):
+            for row in workflow_events:
                 yield "\n"
                 if isinstance(row, dict):
                     dump = json.dumps(row)
@@ -387,13 +408,14 @@ def stream_node_route() -> Response:
     def generator() -> Generator[str, None, None]:
         yield json.dumps(vembda_initiated_event.model_dump(mode="json"))
 
-        for index, row in enumerate(node_events()):
+        for row in node_events():
             yield "\n"
             yield json.dumps(row)
 
     headers = {
         "X-Vellum-SDK-Version": vembda_initiated_event.body.sdk_version,
         "X-Vellum-Server-Version": vembda_initiated_event.body.server_version,
+        "X-Vellum-Events-Emitted": str(_events_emission_enabled(context)),
     }
 
     resp = Response(
@@ -405,6 +427,47 @@ def stream_node_route() -> Response:
     return resp
 
 
+@bp.route("/serialize", methods=["POST"])
+def serialize_route() -> Response:
+    data = request.get_json()
+
+    files = data.get("files", {})
+
+    if not files:
+        return Response(
+            json.dumps({"detail": "No files received"}),
+            status=400,
+            content_type="application/json",
+        )
+
+    try:
+        # Generate a unique namespace for this serialization request
+        namespace = "".join(random.choice(string.ascii_letters + string.digits) for i in range(14))
+        virtual_finder = VirtualFileFinder(files, namespace)
+
+        try:
+            sys.meta_path.append(virtual_finder)
+            exec_config = BaseWorkflowDisplay.serialize_module(namespace).exec_config
+
+            return Response(
+                json.dumps({"exec_config": exec_config}),
+                status=200,
+                content_type="application/json",
+            )
+
+        finally:
+            if virtual_finder in sys.meta_path:
+                sys.meta_path.remove(virtual_finder)
+
+    except Exception as e:
+        logger.exception(f"Error during serialization: {str(e)}")
+        return Response(
+            json.dumps({"detail": f"Serialization failed: {str(e)}"}),
+            status=500,
+            content_type="application/json",
+        )
+
+
 @bp.route("/version", methods=["GET"])
 def get_version_route() -> tuple[dict, int]:
     resp = get_version()
@@ -464,6 +527,11 @@ def get_workflow_request_context(data: dict) -> WorkflowExecutorContext:
         "request_start_time": time.time_ns(),
     }
 
+    if has_request_context():
+        api_version_header = request.headers.get("x-api-version")
+        if api_version_header:
+            context_data["api_version"] = api_version_header
+
     return WorkflowExecutorContext.model_validate(context_data)
 
 
@@ -474,6 +542,11 @@ def get_node_request_context(data: dict) -> NodeExecutorContext:
         "request_start_time": time.time_ns(),
     }
 
+    if has_request_context():
+        api_version_header = request.headers.get("x-api-version")
+        if api_version_header:
+            context_data["api_version"] = api_version_header
+
     return NodeExecutorContext.model_validate(context_data)
 
 
