@@ -12,7 +12,7 @@ from threading import Event as ThreadingEvent
 import time
 from traceback import format_exc
 from uuid import UUID, uuid4
-from typing import Any, Callable, Generator, Iterator, Optional, Tuple, Type
+from typing import TYPE_CHECKING, Any, Callable, Generator, Iterator, Optional, Tuple, Type
 
 from pebble import concurrent
 from vellum_ee.workflows.display.workflows import BaseWorkflowDisplay
@@ -47,6 +47,15 @@ from workflow_server.core.workflow_executor_context import (
 )
 from workflow_server.utils.log_proxy import redirect_log
 
+if TYPE_CHECKING:
+    # Imported for typing only to avoid runtime dependency issues
+    from vellum.workflows.events.emitters.base import BaseWorkflowEmitter
+
+try:
+    from vellum.workflows.events.emitters.vellum import VellumEmitter as _VellumEmitter
+except Exception:
+    _VellumEmitter = None
+
 logger = logging.getLogger(__name__)
 
 
@@ -88,10 +97,15 @@ def _stream_workflow_wrapper(
     executor_context: WorkflowExecutorContext,
     queue: Queue,
     cancel_signal: Optional[ThreadingEvent],
+    timeout_signal: ThreadingEvent,
 ) -> None:
     span_id_emitted = False
     try:
-        stream_iterator, span_id = stream_workflow(executor_context=executor_context, cancel_signal=cancel_signal)
+        stream_iterator, span_id = stream_workflow(
+            executor_context=executor_context,
+            cancel_signal=cancel_signal,
+            timeout_signal=timeout_signal,
+        )
 
         queue.put(f"{SPAN_ID_EVENT}:{span_id}")
         span_id_emitted = True
@@ -119,6 +133,7 @@ def stream_workflow_process_timeout(
     executor_context: WorkflowExecutorContext,
     queue: Queue,
     cancel_signal: Optional[ThreadingEvent],
+    timeout_signal: ThreadingEvent,
 ) -> Process:
     workflow_process = Process(
         target=_stream_workflow_wrapper,
@@ -126,6 +141,7 @@ def stream_workflow_process_timeout(
             executor_context,
             queue,
             cancel_signal,
+            timeout_signal,
         ),
     )
     workflow_process.start()
@@ -150,6 +166,7 @@ def stream_workflow_process_timeout(
 
 def stream_workflow(
     executor_context: WorkflowExecutorContext,
+    timeout_signal: ThreadingEvent,
     disable_redirect: bool = True,
     cancel_signal: Optional[ThreadingEvent] = None,
 ) -> tuple[Iterator[dict], UUID]:
@@ -226,6 +243,7 @@ def stream_workflow(
             executor_context=executor_context,
             stream_generator=call_workflow,
             disable_redirect=disable_redirect,
+            timeout_signal=timeout_signal,
         ),
         stream.span_id,
     )
@@ -265,12 +283,14 @@ def stream_node(
         executor_context=executor_context,
         stream_generator=call_node,
         disable_redirect=disable_redirect,
+        timeout_signal=ThreadingEvent(),
     )
 
 
 def _call_stream(
     executor_context: BaseExecutorContext,
     stream_generator: Callable[[], Generator[dict[str, Any], Any, None]],
+    timeout_signal: ThreadingEvent,
     disable_redirect: bool = True,
 ) -> Iterator[dict]:
     log_redirect: Optional[StringIO] = None
@@ -291,6 +311,7 @@ def _call_stream(
                 log=log_redirect.getvalue() if log_redirect else "",
                 stderr="",
                 container_overhead_latency=executor_context.container_overhead_latency,
+                timed_out=timeout_signal.is_set(),
             ),
             parent=None,
         )
@@ -319,13 +340,35 @@ def _create_workflow(executor_context: WorkflowExecutorContext, namespace: str)
     VembdaExecutionFulfilledEvent.model_rebuild(
         # Not sure why this is needed, but it is required for the VembdaExecutionFulfilledEvent to be
         # properly rebuilt with the recursive types.
+        # use flag here to determine which emitter to use
         _types_namespace={
             "BaseWorkflow": BaseWorkflow,
             "BaseNode": BaseNode,
         },
     )
 
-    return Workflow(context=workflow_context, store=EmptyStore())
+    # Determine whether to enable the Vellum Emitter for event publishing
+    use_vellum_emitter = bool((executor_context.feature_flags or {}).get("vembda-event-emitting-enabled"))
+
+    emitters: list["BaseWorkflowEmitter"] = []
+    if use_vellum_emitter:
+        if _VellumEmitter is not None:
+            try:
+                emitters = [_VellumEmitter()]
+            except Exception:
+                emitters = []
+        else:
+            # Emitter is not available at import time; continue without it
+            logger.warning("VellumEmitter not available; continuing without it")
+
+    # Explicit constructor call to satisfy typing
+    workflow = Workflow(
+        context=workflow_context,
+        store=EmptyStore(),
+        emitters=emitters or None,
+    )
+
+    return workflow
 
 
 def _create_workflow_context(executor_context: BaseExecutorContext) -> WorkflowContext:
@@ -352,6 +395,7 @@ def _create_workflow_context(executor_context: BaseExecutorContext) -> WorkflowC
         vellum_client=Vellum(
             api_key=executor_context.environment_api_key,
             environment=environment,
+            api_version=executor_context.api_version,
         ),
         execution_context=executor_context.execution_context,
         generated_files=executor_context.files,
